/***************************************************************************
                          basisgraph.h  -  description
                             -------------------
    begin                : Thu June 29 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BASISGRAPH_H
#define BASISGRAPH_H

#include <string>
#include <vector>
#include <list>
#include <map>
#include <iostream>

#include "node.h"
#include "edge.h"

namespace graphlib {
typedef map<Node,unsigned>  NodeNumberMap;
class Matrix;

/**
  *@short An abstract basis class for graphs
  *
  * A graph <var>G=(V,E)</var> consists of a set of nodes <var>V</var>
  * and a set of edges <var>E</var>
  * The graph is not the owner of the nodes and edges
  *
  * @see Edge
  * @see Node
  * @see Graph
  */

class BasisGraph
{
    //------------------------------------------------------ friends
    /**
      * Output operator.
      * Creates an GML output of the <code>g</code>.
      *
      * @param os the output-stream
      * @param g the basisgraph
      * @see <a href="http://www.infosun.fmi.uni-passau.de/Graphlet/
      *       GML/index.html">The GML File Format</a>
      * @return the output-stream
      */
    friend ostream& operator<< (ostream& os, const BasisGraph& g);

    /**
      * Output operator.
      * Writes the nodes and edges of <code>g</code>.
      *
      * @param os the output-stream
      * @param g the basisgraph
      */
    friend ostream& operator<< (ostream& os, const BasisGraph* g);

  public:
    //------------------------------------------------------ Con-/Destructors
    /**
      * Constructor.
      * Generates an empty BasisGraph, without any nodes and any edges,
      */
    BasisGraph(int number=0);

    /**
      * Copy Constructor.
      * Copies the list of nodes and edges
      */
    BasisGraph(const BasisGraph&);

    /**
      * Destructor. Nothing to do.
      */
    virtual ~BasisGraph() { ; }

    //------------------------------------------------------ Operators
    /**
      * Assignement Operator
      */
    BasisGraph& operator=(const BasisGraph& g);

  public:
    //------------------------------------------------------ Information
    /**
      * Returns the <var>index</var> of the graph.
      */
    int index() const  { return number_; }

    /**
      * Sets the <var>index</var> of the graph.
      *
      * @param number
      */
    void index(int number) { number_= number; }

    /**
      * Returns the number of nodes in the graph.
      *
      * @return number of nodes
      */
    unsigned numberNodes() const { return nodes.size(); }

    /**
      * Returns the number of edges in the graph.
      *
      * @return number of edges
      */
    unsigned numberEdges() const { return edges.size(); }

    /**
      * Creates a map where the nodes are assoziated with
      * unsigned numbers.and stores it in <code>nodeMap</code>.
      */
    const NodeNumberMap& nodeNumberMap() const { return nodeMap; }

    /**
      * Makes the adjacency matrix of the graph.
      * <em>Note</em>: Allocates new memory.
      */
    Matrix* makeAdjacency() const;

    /**
      * Makes the degree matrix of the graph.
      * <em>Note</em>: Allocates new memory.
      */
    Matrix* makeDegree() const;


    //------------------------------------------------------ Deletion
    /**
      * Clears list of nodes and edges. Does not delete them !!!
      */
    virtual void clear();

    //------------------------------------------------------ Iterators
    /**
      * Iterate through all nodes in the graph.
      *
      * @return start for iteration through all nodes in the graph.
      */
    constNodeIter nodesBegin() const { return nodes.begin(); }

    /**
      * Iterate through all nodes in the graph.
      *
      * @return start for iteration through all nodes in the graph.
      */
    NodeIter nodesBegin() { return nodes.begin(); }

    /**
      * Iterate through all nodes in the graph.
      *
      * @return end for iteration through all nodes in the graph.
      */
    constNodeIter nodesEnd() const { return nodes.end(); }

    /**
      * Iterate through all nodes in the graph.
      *
      * @return end for iteration through all nodes in the graph.
      */
    NodeIter nodesEnd() { return nodes.end(); }

    /**
      * Iterate through all edges in the graph.
      *
      * @return start for iteration through all edges in the graph.
      */
    constEdgeIter edgesBegin() const { return edges.begin(); }

    /**
      * Iterate through all edges in the graph.
      *
      * @return start for iteration through all edges in the graph.
      */
    EdgeIter edgesBegin() { return edges.begin(); }

    /**
      * Iterate through all edges in the graph.
      *
      * @return end for iteration through all edges in the graph.
      */
    constEdgeIter edgesEnd() const { return edges.end(); }

    /**
      * Iterate through all edges in the graph.
      *
      * @return end for iteration through all edges in the graph.
      */
    EdgeIter edgesEnd() { return edges.end(); }

    //------------------------------------------------------ Search
    /**
      * Returns the position in the list of all nodes of node with
      * <code>id</code>. If there's no node with <code>id</code>
      * the end will be returned.
      *
      * @param id of a valid node
      * @return const_iterator to the node or end
      */
    constNodeIter findNode(unsigned id) const;

    /**
      * Returns the position in the list of all nodes of node with
      * <code>id</code>. If there's no node with <code>id</code>
      * the end will be returned.
      *
      * @param id of a valid node
      * @return iterator to the node or end
      */
    NodeIter findNode(unsigned id);

    /**
      * Returns the position in the list of all nodes of first node
      * with id greater than <code>id</code>.
      * If there's no node with such node
      * the end will be returned.
      *
      * @param id of a valid node
      * @return iterator to the node or end
      */
    NodeIter findGreaterNode(unsigned id);

    /**
      * Returns the position in the list of all edges of an edge
      * with <code>id</code>. If there's no edge with <code>id</code>
      * the end will be returned.
      *
      * @param <code>id</code> id of a valid edge
      * @return const_iterator to the edge or end
      */
    constEdgeIter findEdge(unsigned id) const;

    /**
      * Returns the position in the list of all edges of an edge
      * with <code>id</code>. If there's no edge with <code>id</code>
      * the end will be returned.
      *
      * @param <code>id</code> id of a valid edge
      * @return iterator to the edge or end
      */
    EdgeIter findEdge(unsigned id);

    /**
      * Returns the position in the list of all edges of an edge
      * with <code>s</code>ource and <code>t</code>arge. If there's no
      * edge with <code>s</code> and <code>t</code> the end will
      * be returned.
      *
      * @param <code>s</code> source of a valid edge
      * @param <code>t</code> target of a valid edge
      * @return const_iterator to the edge or end
      */
    constEdgeIter findEdge(unsigned s, unsigned t) const;

    /**
      * Returns the position in the list of all edges of an edge
      * with <code>s</code>ource and <code>t</code>arge. If there's no
      * edge with <code>s</code> and <code>t</code> the end will
      * be returned.
      *
      * @param <code>s</code> source of a valid edge
      * @param <code>t</code> target of a valid edge
      * @return iterator to the edge or end
      */
    EdgeIter findEdge(unsigned s, unsigned t);

  protected:
    //------------------------------------------------------ Nodes/Edges
    // number of the graph
    int               number_;

    //------------------------------------------------------ Nodes/Edges
    /// list of nodes
    list<Node>        nodes;
    /// list of edges
    list<Edge>        edges;

    /// map Node<->unsigned
    mutable NodeNumberMap     nodeMap;

  private:
    //------------------------------------------------------ functions
    /// initialize map: Node <-> unsigned
    void initNodeNumberMap() const;



};  // class BasisGraph

} // namespace graphlib

#endif // BASISGRAPH_H

/* end of file */
