/***************************************************************************
                          bicomponents.h  -  description
                             -------------------
    begin                : Thu Mar 8 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BICOMPONENTS_H
#define BICOMPONENTS_H

#include "graph.h"
#include "component.h"
#include "bicompdata.h"
#include "graphlib_error.h"
using namespace graphlibError;

#include <list>
#include <vector>
#include <map>
#include <set>
#include <iostream>

namespace graphlib {

typedef vector<list<Node> >::const_iterator cVecListNodeIter;
typedef vector<list<Edge> >::const_iterator cVecListEdgeIter;


/**
  * @short Connected and Biconnected Components of an undirected graph
  *
  * Biconnected components are maximal beconnected subgraphs of a given graph.
  * @see Graph
  */

class BiComponents
{
    //------------------------------------------------------ typedefs
    typedef map<Node,BiCompData,less<Node> >     NodeMap;
    typedef map<Edge,bool,less<Edge> >           EdgeMap;
    typedef set<Node>                            NodeSet;

    //------------------------------------------------------ friend functions
    /**
      * Output-operator.
      * Creates an GML output of the components. Each of them is written
      * as a GML graph.
      *
      * @param os the output-stream
      * @param bc the biconnected components
      * @see <a href="http://www.infosun.fmi.uni-passau.de/Graphlet/
      *       GML/index.html">The GML File Format</a>
      * @return the output-stream
      */
    friend ostream& operator<<(ostream&,const BiComponents&);

  public:
    //------------------------------------------------------ Con-/Destructors
    /**
      * Constructor: Creates all biconnected components <code>g</code>.
      *
      * For each component a new Component is created.
      * @see Component
      *
      * @param <code>g</code> the graph
      */
    BiComponents(const Graph* g)
        throw(EmptyGraphError,logic_error,EmptyNodeError);

    /**
      * Destructor: Deletes all components.
      */
    ~BiComponents();

    //------------------------------------------------------ Information
    /**
      * Returns the number of biconnected components.
      */
    unsigned numberBiComp() const { return bicomp.size(); }

    /**
      * Returns the Component-list of biconnected components.
      */
    const list<Component*>& biComponentsSubGraphs() const
          { return bicomp; }

    /**
      * Returns the number of connected components.
      */
    unsigned numberConComp() const { return comp.size(); }

    /**
      * Returns of the Component-list of connected components.
      */
    const list<Component*>& conComponentsSubGraphs() const
          { return comp; }

    //------------------------------------------------------ Output
    /**
      * writes to ostream the nodeMap and edgeMap.
      */
    void map_ausgabe(ostream&) const;

    /**
      * writes to ostream the connected and biconnected
      * components as lists of nodes.
      */
    void writeComponentsNodes(ostream&) const;

    /**
      * writes to ostream the connected and biconnected
      * belonging to this connected components as list of nodes.
      */
    void writeCompBiCompNodes(ostream& os) const;

  private:
    //------------------------------------------------------ Data
    /// graph to which these components belong
    const Graph*              graph;
    /// biconnected components
    list<Component*>           bicomp;
    /// connected components
    list<Component*>           comp;
    /// set of all unvisited nodes
    NodeSet                   nodeSet;
    /// map Node <-> BiCompData
    NodeMap                   nodeMap;
    /// map Edge <-> bool (visited)
    EdgeMap                   edgeMap;

    //------------------------------------------------------ Utilities
    /**
      * Algorithm: finds all connected and biconnected components
      * and stores them in the lists of graphs and lists of nodes
      */
    void graph2BiconnComp() throw(logic_error,EmptyNodeError);

    /**
      * Initialize the NodeMap and the NodeSet
      */
    void makeNodeMap();

    /**
      * Initialize the EdgeMap
      */
    void makeEdgeMap();

    /**
      * Find next unvisited edge incident to node <code>n</code>
      *
      * @return Iterator to found edge
      */
    constEdgeIter findUnvisitedEdge(const Node& n) const
          throw(EmptyNodeError);

   /**
      * Creates from all the edges from the top of <code>edge_stack</code>
      * up to the edge (<code>father</code>,<code>v</code>) a new graph.
      * Pushes this graph into components and removes all used edges from
      * the <code>edge_stack</code>. If <code>all</code> is true, all
      * edges will be used.
      * New biconnected component is added to list of components.
      * Furthermore creates a new list of nodes for this component and
      * adds it into the list of components.
      *
      * @param edge_stack stack with edges.
      * @param father
      * @param v
      * @param biNr number of biconnected component
      * @param all
      */
    void stack2Component(list<Edge>& edge_stack,const Node& father, const Node& v,
                         unsigned biNr, bool all=false);


    /**
      * Sets for all components in the list beginning with <code>it</code> till <code>end</code>
      * the component to which the belong to <code>conn</code>.
      *
      * @param conn the component to which the components in the list belongs
      * @param it the start of the list
      * @param end the end of the list (outside)
      */
    void setComponents(const Component* conn, list<Component*>::iterator it,
                       list<Component*>::iterator end);




}; // class BiComponents


} // namespace graphlib
#endif // BICOMPONENTS_H

/* end of file */
