/***************************************************************************
                          bitvector.cpp  -  description                              
                             -------------------                                         
    begin                : Tue Dec 7 1999                                           
    copyright            : (C) 1999 by Gleiss Petra                         
    email                : petra@gleiss.at                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "bitvector.h"
#include "trace.h"
#include "graphlib_error.h"
using namespace graphlibError;

#include <string>

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
BitVector::BitVector(unsigned asize) : size(asize), length(0)
{
  if (asize == 0)
  {
    words = 0;
    buffer = 0;
  }

  else
  {
    words = calcBytes();
    buffer = new Bit_Vec[words];
    for (unsigned i=0; i!= words; i++) buffer[i]=0x0;
  }
}

BitVector::BitVector(const BitVector& aBV) : buffer(0)
{
  copy(aBV);
}
BitVector::BitVector(const BitVector* aBV) : buffer(0)
{
  copy(*aBV);
}

BitVector::~BitVector()
{
  if (buffer) delete [] buffer;
  buffer = 0;
}

//---------------------------------------------------------------------------
// private functions
//---------------------------------------------------------------------------
unsigned BitVector::calcBytes() const
{
  return (((size%BITS)==0 ? 0 : 1 ) + size/BITS);
}

unsigned BitVector::getoffset(int pos) const
{
  return (BITS-1-pos%BITS);
}

void BitVector::copy(const BitVector& aBV)
{
  length = aBV.length;
  size   = aBV.size;
  words  = aBV.words;

  if(aBV.size == 0)
    buffer = 0;

  else
  {
    // delete old buffer
    if (buffer) delete [] buffer;

    // assign buffer
    buffer=new Bit_Vec[words];
    for(unsigned i=0; i<words; i++)
      buffer[i] = aBV.buffer[i];
  }
}

//---------------------------------------------------------------------------
// Operators
//---------------------------------------------------------------------------
BitVector& BitVector::operator=(const BitVector& aBV)
{
  if (this == &aBV) return *this;
  copy(aBV);
  return *this;
}

BitVector& BitVector::operator=(const char* abuffer)
{
  string error="BitVector::operator=(const char*): ";
  clearAllBits();
  unsigned i=0, pos=0;
  while (abuffer[i] && pos<size)
  {
    switch (abuffer[i])
    {
    case '1':
      setBit(pos++); i++; break;
    case '0':
      pos++; i++; break;
    case ' ':
      i++; break;
    default:
      TRACE(error+"strange char ("+abuffer[i]+") -> ignored\n")
      i++;
    }
  }

  // remaining characters
  if(abuffer[i])
    TRACE(error+"to long -> remaining ignored\n")

  return *this;
}

bool BitVector::operator==(const BitVector& aBV) const throw(SizeError)
{
  if (size!=aBV.size)
    THROW(SizeError,"BitVector::operator== (const BitVector&) const")

  if (count()!=aBV.count())
    return false;

  for (unsigned i=0; i<words; i++)
    if (buffer[i]!=aBV.buffer[i])
      return false;
  return true;
}

bool BitVector::operator!=(const BitVector& aBV) const throw(SizeError)
{
  return !(*this == aBV);
}

bool BitVector::operator<(const BitVector& aBV) const throw(SizeError)
{
  if (size!=aBV.size)
    THROW(SizeError,"BitVector::operator< (const BitVector&) const")

  return (count() < aBV.count() ? true : false );
}

bool BitVector::operator>(const BitVector& aBV) const throw(SizeError)
{
  if (size!=aBV.size)
    THROW(SizeError,"BitVector::operator> (const BitVector&) const")

  return (count() > aBV.count() ? true : false );
}

BitVector BitVector::operator&(const BitVector& aBV) const throw(SizeError)
{
  if (size!=aBV.size)
    THROW(SizeError,"BitVector::operator& (const BitVector&) const")

  BitVector b(size);
  for (unsigned i=0; i<words; i++)
    b.buffer[i] = buffer[i] & aBV.buffer[i];
  b.length = -1;
  return b;
}

BitVector BitVector::operator|(const BitVector& aBV) const throw(SizeError)
{
  if (size!=aBV.size)
    THROW(SizeError,"BitVector::operator| (const BitVector&) const")

  BitVector b(size);
  for (unsigned i=0; i<words; i++)
    b.buffer[i] = buffer[i] | aBV.buffer[i];
  b.length = -1;
  return b;
}

BitVector BitVector::operator^(const BitVector& aBV) const throw(SizeError)
{
  if (size!=aBV.size)
    THROW(SizeError,"BitVector::operator^ (const BitVector&) const")

  BitVector b(size);
  for (unsigned i=0; i<words; i++)
    b.buffer[i] = buffer[i] ^ aBV.buffer[i];
  b.length = -1;
  return b;
}

BitVector& BitVector::operator&=(const BitVector& aBV) throw(SizeError)
{
  if (size!=aBV.size)
    THROW(SizeError,"BitVector::operator&= (const BitVector&)")

  for (unsigned i=0; i<words; i++)
    buffer[i] &= aBV.buffer[i];
  length = -1;
  return *this;
}

BitVector& BitVector::operator|=(const BitVector& aBV) throw(SizeError)
{
  if (size!=aBV.size)
    THROW(SizeError,"BitVector::operator|= (const BitVector&)")

  for (unsigned i=0; i<words; i++)
    buffer[i] |= aBV.buffer[i];
  length = -1;
  return *this;
}

BitVector& BitVector::operator^=(const BitVector& aBV) throw(SizeError)
{
  if (size!=aBV.size)
    THROW(SizeError,"BitVector::operator^= (const BitVector&)")

  for (unsigned i=0; i<words; i++)
    buffer[i] ^= aBV.buffer[i];
 length = -1;
 return *this;
}

void BitVector::AND_Negation(const BitVector& aBV) throw(SizeError)
{
  if (size!=aBV.size)
    THROW(SizeError,"BitVector::operator&= (const BitVector&)")

  for (unsigned i=0; i<words; i++)
    buffer[i] &= ~(aBV.buffer[i]);
  length = -1;
}

bool BitVector::greaterValue(const BitVector& aBV) const throw(SizeError)
{
  if (size!=aBV.size)
    THROW(SizeError,"BitVector::greaterValue(const BitVector&) const")

  int i=0;
  Bit_Vec b1;
  Bit_Vec b2;
  do
  {
    b1=buffer[i];
    b2=aBV.buffer[i];
    if (b1<b2)
      return false;
    else if (b1>b2)
      return true;
    i++;
  } while (i<length);

  return false;
}

//---------------------------------------------------------------------------
// Input / Output
//---------------------------------------------------------------------------
ostream& operator<< (ostream& os, const BitVector& aBV)
  throw(logic_error,BufferError,OutsideError)
{
  unsigned i;
  os << aBV.getsize() << "  " << aBV.count() << "\t";

  // the bitfield itself
  for (i=0;i<aBV.getsize();i++)
  {
    if ((i%4)==0) os << " ";
    if (i%BITS==0 && i!=0) os << "| ";
    if (aBV.getBit(i))
      os << "1";
    else
      os << "0";
  }
/*  os << " (";
  for (i=aBV.getsize();i!=BITS*aBV.getwords();i++) {
    if ((i%4)==0) os << " ";
    os << (aBV.getbuffer()[i/BITS] & (0x1 << BITS-1-i%BITS));
  }
  os << ")";
*/
  os << endl;
  return os;
}

ostream& operator<< (ostream& os, const BitVector* aBV)
  throw(logic_error,BufferError,OutsideError)
{
  unsigned i;
  for (i=0;i<aBV->getsize();i++)
  {
    if ((i%4)==0)
      os << " ";
    if (aBV->getBit(i))
      os << "1";
    else
      os << "0";
  }
  return os;
}


//---------------------------------------------------------------------------
// Information
//---------------------------------------------------------------------------
unsigned BitVector::getsize() const throw(logic_error)
{
  if (size==0)
    THROW(logic_error,"BitVector::getsize() const: BitVector == 0")
  return size;
}

unsigned BitVector::getwords() const throw(logic_error)
{
  if (words==0)
    THROW(logic_error,"BitVector::getwords() const: BitVector == 0")
  return words;
}

Bit_Vec* BitVector::getbuffer() const
{
  return buffer;
}

void BitVector::setLength() const throw(BufferError)
{
  if (!buffer)
    THROW(BufferError,"BitVector::setlength() const")

  length = 0;
  unsigned i, pos;
  for (i=0, pos=0; i!=words; i++)
  {
    if (buffer[i]==0)
      pos += BITS;
    else
      do
      {
        if (getBit(pos++))
          length++;
      } while (pos%BITS && pos<size);
  }
}

int BitVector::count() const throw(BufferError)
{
  if (length<0)
    setLength();

  return length;
}

bool BitVector::getBit(unsigned pos) const
  throw(BufferError,OutsideError)
{
  if (!buffer)
    THROW(BufferError,"BitVector::getBit(int) const")
  if (pos>=size)
    THROW(OutsideError,"BitVector::getBit(int) const: pos > size")

  int i = pos/BITS;
  return (buffer[i] & (0x1 << getoffset(pos)));
}

bool BitVector::empty() const throw(BufferError)
{
  if (!buffer)
    THROW(BufferError,"BitVector::empty() const")

  for (unsigned i=0; i!=words; i++)
    if (buffer[i])
      return false;
  return true;
}

bool BitVector::setBit(unsigned pos)
  throw(BufferError,OutsideError)
{
  if (!buffer)
    THROW(BufferError,"BitVector::setBit(int)")
  if (pos >= size)
    THROW(OutsideError,"BitVector::setBit(int): pos > size")

  // if Bit set
  if (getBit(pos))
  {
    clearBit(pos);
    return false;
  }

  unsigned i=pos/BITS;
  buffer[i] |=  (0x1 << getoffset(pos));
  length++;
  return true;
}

BitVector& BitVector::setBit(const char* abuffer)
{
  return (*this=abuffer);
}

BitVector& BitVector::setBit(unsigned begin, unsigned end)
  throw(BufferError,OutsideError)
{
  if (!buffer)
    THROW(BufferError,"BitVector::setBit(int,int)")
  if ((begin >= size) || (end > size))
    THROW(OutsideError,"BitVector::setBit(int,int): pos > size")

  unsigned i, pos;
  for (pos=begin; pos<end; pos++)
  {
    i=pos/BITS;
    buffer[i] |=  (0x1 << getoffset(pos));
    length++;
  }
  return *this;
}

BitVector& BitVector::setAllBits()
  throw(BufferError,OutsideError)
{
  if (!buffer)
    THROW(BufferError,"BitVector::setAllBits()")

  for (unsigned i=0; i<words-1;i++)
    buffer[i] = ~0x00;
  if (size%BITS == 0)
    buffer[words-1] = ~0x00;
  else
  {
    unsigned pos = (words-1)*BITS;
    setBit(pos,size-1);
  }
  length = size;
  return *this;
}

BitVector& BitVector::clearBit(unsigned pos)
  throw(BufferError,OutsideError)
{
  if (!buffer)
    THROW(BufferError,"BitVector::clearBit(int)")
  if (pos > size)
    THROW(OutsideError,"BitVector::clearBit(int): pos > size")

  unsigned i=pos/BITS;
  buffer[i] &= ~(0x1 << getoffset(pos));
  length--;
  return *this;
}

BitVector& BitVector::clearBit(unsigned begin, unsigned end)
    throw(BufferError,OutsideError)
{
  if (!buffer)
    THROW(BufferError,"BitVector::clearBit(int,int)")
  if ((begin >= size) || (end > size))
    THROW(OutsideError,"BitVector::clearBit(int,int): pos > size")

  unsigned i, pos;
  for(pos=begin; pos<end; pos++)
  {
    i=pos/BITS;
    buffer[i] &= ~(0x1 << getoffset(pos));
    length--;
  }
  return *this;
}

BitVector& BitVector::clearAllBits() throw(BufferError)
{
  if (!buffer)
    THROW(BufferError,"BitVector::clearAllBits()")

  for (unsigned i=0; i<words;i++)
    buffer[i] = 0x00;
  length=0;
  return *this;
}

} // namespace graphlib

/* end of file */

