/***************************************************************************
                          cycle.h  -  description                              
                             -------------------                                         
    begin                : Wed Dec 22 1999                                           
    copyright            : (C) 1999 by Gleiss Petra                         
    email                : petra@gleiss.at                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef CYCLE_H
#define CYCLE_H

#include "bitvector.h"
#include "subgraph.h"

#include "utilities.h"
#include "graphlib_error.h"
using namespace graphlibError;


/**
  *@author Gleiss Petra
  */

namespace graphlib {

class Cycle : public BitVector,
              public SubGraph
{
    //------------------------------------------------------ friend functions
    /**
      * Output-operator.
      *
      * @param ostream&
      * @param const Cycle&
      * @return ostream&
      */
    friend ostream& operator<< (ostream&, const Cycle&);

    /**
      * Output-operator.
      *
      * @param ostream&
      * @param const Cycle*
      * @return ostream&
      */
    friend ostream& operator<< (ostream&, const Cycle*);

  public:
    //------------------------------------------------------ Con-/Destructors
    /**
      * Constructor.
      */
    Cycle(unsigned asize=0, const Graph* g=0);

    /**
      * Copy Constructor.
      */
    Cycle(const Cycle&);

    /**
      * Destructor.
      */
    virtual ~Cycle() { ; }

    //------------------------------------------------------ Operators
    /**
      * Assignment Operator.
      */
    Cycle& operator=(const Cycle&);

  //  bool operator==(const Cycle&) const;
  //  bool operator!=(const Cycle&) const;

    /**
      * Returns true if index of all cycles is smaller.
      */
    bool lessIndex(const Cycle&) const throw(SizeError);

    /**
      * Returns true if index of prototype is smaller.
      */
    bool lessPrototypeIndex(const Cycle& aCycle) const throw(SizeError);

    /**
      * Returns true if index of interchangeability classes is smaller.
      */
    bool lessXchgClass(const Cycle&) const throw(SizeError);

    //------------------------------------------------------ Information
    /**
      * Sets <var>prototype</var> to <code>index</code>.
      */
    void prototypeIndex (unsigned long index);

    /**
      * Returns index of prototype.
      */
    unsigned long prototypeIndex () const;

    /**
      * Sets <var>xchgClass</var> to <code>index</code>.
      */
    void xchgClassIndex(unsigned long);

    /**
      * Returns index of interchangeability classen.
      */
    unsigned long xchgClassIndex() const;

    /**
      *
      */
    enum MemberFlag {Prototype,
                     Mcb,
                     Relevant,
                     Shortest,
                     UniqueShortest,
                     Essential,
                     Empty,
                     UPath};

  public:
    /**
      * Returns true if the cycle is a <code>flag</code>.
      */
    bool getmembers(MemberFlag flag) const;

    /**
      * Sets the <code>flag</code>.
      */
    void setmembers(MemberFlag flag);

    /**
      * Clears the <code>flag</code>.
      */
    void clearmembers(MemberFlag flag);

    /**
      * Returns the list of nodes in the cycle.
      */
    const list<Node>& cycleNodes() const { return nodes; }

    /**
      * Writes the members to <code>os</code>.
      */
    void write_members(ostream& os) const;

    /**
      * Returns string with membership (1 for ist member, 0 for is not
      * a member of the cycle set).
      */
    string membership() const;

    //------------------------------------------------------ Creation
    /**
      * Adds <code>node</code> to cyclenodes.
      *
      * @param a valid node
      */
    void addNodes(const Node& node);

    /**
      * Adds all nodes from <code>begin</code> to <code>end</code>
      * from the list of nodes to cyclenodes in the same orientation
      * as in the list.
      *
      * @param begin of list of nodes
      * @param end of list of nodes
      */
    void addNodes(NodeIter begin, NodeIter end);
    void addNodes(list<Node>::reverse_iterator begin,
                  list<Node>::reverse_iterator end);

    /**
      * Calculates the cycle form the list of nodes <var>cyclenodes</var>
      * For cycles the first and last node has to be the same !!!
      * Returns false if an edge occures twice.
      *
      * @param EdgeIdMap
      * @return true if no edge twice
      */
    bool calcCycle(const EdgeIdMap&) throw(logic_error,runtime_error);


  protected:
    //------------------------------------------------------ datas
    /// number of the prototype cycle family
    unsigned long prototype;
    /// number of the interchangeability class
    unsigned long xchgClass;
    /**
      * flag of characterictic:
      * 0 ... prototype
      * 1 ... mcb
      * 2 ... relevant
      * 3 ... short
      * 4 ... unique short
      * 5 ... essential
      * 6 ...
      * 7 ... is path
      */
    unsigned char member;

}; // class Cycle

class less_index: public less<Cycle> {
public:
  bool operator()(const Cycle& x, const Cycle& y) const {
    return x.lessIndex(y);
  }
}; // struct  less_index

class less_prototypeindex: public less<Cycle> {
public:
  bool operator()(const Cycle& x, const Cycle& y) const {
    return x.lessPrototypeIndex(y);
  }
}; // struct  less_prototypeindex

class less_xchgClass: public less<Cycle> {
public:
  bool operator()(const Cycle x, const Cycle y) const {
    return x.lessXchgClass(y);
  }
}; // struct  less_xchgClass

} // namespace graphlib

#endif // CYCLE_H

/* end of file */
