/***************************************************************************
                          cyclegraph.h  -  description
                             -------------------
    begin                : Mon May 21 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CYCLEGRAPH_H
#define CYCLEGRAPH_H

#include "graphlib.h"
#include "graph.h"
#include "component.h"

#include "nodepath.h"

#include "prototype.h"
#include "cycle.h"

#include <cmath>

#include <vector>
#include <map>
#include <set>

namespace graphlib {

typedef map<Node,NodePath,less<Node> >        NodePathMap;
typedef set<Node>                             NodeSet;

typedef vector<Component*>::iterator           SubGraphIter;
typedef vector<Component*>::const_iterator     constSubGraphIter;

typedef list<Cycle*>::iterator                CycleIter;
typedef list<Cycle*>::const_iterator          constCycleIter;

typedef vector<Cycle*>::iterator              CycleVIter;
typedef vector<Cycle*>::const_iterator        constCycleVIter;

/**
  *@short An undirected graph
  *
  * A graph for calculating cycle decompositions.
  * Contains functions
  * for the calculation of the minimal cycle basis, the
  * relevant, essential, shortest and unique shortest cyles and the
  * interchangeability classes. These calculations are although
  * possible over the vector space containing the u-paths and cycles,
  * if a list of U-Nodes is set.
  *
  * @see Graph
  * @see
  *   Gleiss, P.M., Leydold, J. and Stadler, P.F. Interchangeability of
  *     Relevant Cycles in Graphs, Elec. J. Comb. 7, R16 (2000)
  *   Vismara, P Union of all the minimum cycle bases of a graph,
  *     Elec. J. Comb. 4, R9 (1997)
  *   <a href="http://www.combinatorics.org/">Elec. J. Comb.</a>
  */

class CycleGraph : virtual public Graph
{

  public:
    //------------------------------------------------------ Con-/Destructors
    /**
      * Constructor.
      */
    CycleGraph();

    /**
      * Destructor.
      * Calls clearCalculations();
      */
    virtual ~CycleGraph();

    //------------------------------------------------------ Deletion
    /**
      * Calls clearCalculations();
      * Calls Graph::clear();
      */
    virtual void clear();

  private:
    //------------------------------------------------------ Operators
    /**
      * CopyConstructor -> not implemented
      */
    CycleGraph(const CycleGraph&);
    /**
      * Assignment Operator -> not implemented
      */
    CycleGraph& operator=(const CycleGraph&);

  protected:
    /**
      * Deletes the biconnected components, the digraphs, cycles and u-paths.
      * Empties the U-nodes and edgeMap and sets CalcStat = NO.
      */
    virtual void clearCalculations();

  public:
    //------------------------------------------------------ Information
     /**
       * Returns the cycles of the graph.
       */
     const vector<Cycle*>& relevantCycles() const { return cycleVec; }

     /**
       * Returns the cycles and u-paths of the graph.
       */
     const vector<Cycle*>& relevantUElements() const { return uElemVec; }
     
     /**
       * Returns the number of connected components.
       */
     int connectedComponents() const { return components; }

     /**
       * Returns the number of biconnected components.
       */
     int biconnectedComponents() const { return bicomponents; }

     //------------------------------------------------------ Calculations
     /**
       * Calculates connected and biconnected components.
       */
     void calcComponents();

     /**
       * Computes the cycle basis dependend on <code>flag</code>.
       * Horton, Prototypes, Relevants (default), InterchangeabilityClasses
       * Essentials, Unique_Shortest.
       *
       * @param flag
       */
     void calcCycleBasis(DecompositionsMode flag=All);

     /**
       * Computes u-basis dependend on <code>flag</code>.
       * Horton, Prototypes, Relevants (default), InterchangeabilityClasses
       * Essentials, Unique_Shortest.
       *
       * @param flag
       */
    void calcUBasis(DecompositionsMode flag=All);

    /**
      * Makes a statistic:
      * Cycle length and number of cycles.
      * Writes the statistic to file named <code>name.cycle_stat</code>.
      *
      * @param name
      */
    void makeCycleStatistic(string name);

    /**
      * Returns true, if the cycles with <code>indeces</code>
      * are independent.
      */
    bool indepTest(const list<int>& indeces);

    //------------------------------------------------------ Set Funtions
    /**
      * Sets the U-nodes given in <code>unodes</code>.
      * (ids seperated with spaces)
      *
      * @param unodes file containing list of U-Nodes.
      */
    void setUnodes(string unodes);

    /**
      * Sets the U-nodes from <code>unodes</code>.
      *
      * @param vector<int>& list of U-Nodes.
      */
    void setUnodes(const vector<int>& unodes);

  private:
   //------------------------------------------------------ Deletion
   /// deletes all cycles
   void deleteCycles();

   /// deletes all cycles and u-paths
   void deleteUElements();

   /// deletes all digraphs
   void deleteDiGraphs();

   /// deletes all digraphs for the unodes
   void deleteUnodeDiGraphs();

   /// deletes all biconnected components
   void deleteBiComp();

   /// deletes all components
   void deleteComp();

    //------------------------------------------------------ Calculations
    /**
      * Calculates the all shortest pathes from <code>node</code>
      * to all other nodes of the biconnected components represented
      * by <code>set</code> of this graph.
      * The path contains the starting and the end vertices.
      * If <code>ordered</code> is true
      * (\em default), than only pathes to preceding numbers are
      * calculated, else all shortest pathes are calculated.
      * Stores the pathes in a the NodePathMap <code>map</code> and returns
      * the map
      *
      * @param Node
      * @param NodeSet
      * @param NodePathMap
      * @param ordered
      * @return NodePathMap
      */
    NodePathMap& dijkstra(Node, NodePathMap& map, NodeSet& set,
                          bool ordered) const;

    /**
      * Initialize <code>map</code>:
      * clears path and pushes all nodes of the Component <code>biComp</code>
      * into it, set distance to INf
      * and marked to false for all nodes except for <code>node</code>,
      * for <code>node</code> distance = 0.
      * Initialize <code>set</code>:
      * pushes all nodes of the list of nodes <code>biComp</code> into it.
      *
      * @param Node
      * @param biComp
      * @param NodePathMap (will be cleared)
      * @param NodeSet (will be cleared)
      */
    void initNodeMap(const Node& node, const Component& biComp,
                     NodePathMap& map, NodeSet& set) const;

    /**
      * Returns a node from the <code>nodeSet</code>
      * with the smallest distance found in <code>nodeMap</code>.
      *
      * @param const NodePathMap
      * @param const NodeSet
      * @return Node
      */
    Node findNodeWithSmallestDistance(const NodePathMap& nodeMap,
         const NodeSet&) const;

    /**
      * Computes from a biconnected Component <code>biComp</code>,
      * given as Component of this graph
      * the set C'I including one cycle for each triple
      * r, p, q (bzw. quadruple r, p, q, x), satisfying the following
      * condition:
      * /par
      * only for graphs with weight of all the edges = 1 !!!
      * any relevant cycle C consists of two disjoint shortest
      * paths (r-p) and (r-q) linked by the edge
      * (p,q) if C is odd or by the path (p,x,q) if C is even.
      * see Visamra (1997) pp 8-10 algorithm 1
      *
      * Stores the cycles in <code>prototypes</code>.
      * Stores the digraphs D_r_i = {V_r,U_r) needed for generating
      * the cycle families in <code>digraphs</code>.
      * U_r = { directed edges(y,z) such that | (z,y) belongs to
      * a shortest path in G from r to y that passes only through
      * vertices which precede r in the ascending ordering of
      * the vertices ids}.
      *
      * @param biComp a biconnected component as Component
      * @param prototypes an empty list where the cycles will be
      *       stored
      */
    void calcCI(const Component& biComp, list<Prototype>& prototypes);

    /**
      * Computes the prototypes of relevant U-pathes
      *
      * see Gleiss - Diss
      *
      * @param Comp a component as Component
      * @param unodes in this component
      * @param prototypes a list where the u-pathes will be stored
      */
    void calcUpathes(const Component& Comp, const list<Node>& unodes,
        list<Prototype>& prototypes);

    /**
      * Computes the set of prototypes of relevant cycles and u-pathes
      * by extraction form a list of cycles/u-pathes containing them
      * Extracts all relevant cycles and u-pathes of C'R
      * and partitions them into cycles families (= prototypes)
      * see Vismara (1997) pp 10-11
      *
      * If only cycles are calculates, the algorithm stops when
      * <code>cyclom</code> number of cycles found for the mcb. For
      * the relevants it stops after the length for the last
      * cycle from the mcb.
      *
      * For calculating cycles and u-pathes set <code>cyclom</code> = -1.
      *
      * Stores the cycles in <code>prototypes</code>, assigning them
      * a prototypeInex starting by <code>index</code>.
      *
      * If <code>flag</code> is Horton, extracts a minimal cycles basis
      * and stores it in the cycles.
      * If <code>flag</code> is Prototypes, stores the prototypes in the
      * cycles.
      *
      * @param prototypes
      * @param index of the next prototype
      * @param cyclomatic number
      */
    void calcPrototypes(list<Prototype>& prototypes, unsigned long& index,
              int cyclom, DecompositionsMode flag);

    /**
      * Computes the union of all the minimum cycle basis
      * extraction of all relevant cycles of CR (cycle)
      * see Vismara (1997) pp 10-11
      *
      * Stores the cycles in <var>cycles</var> and <code>relevants</code>.
      * Extracts an minimum cycle basis and stores it in <code>mcb</code>.
      *
      * @param prototypes
      * @param relevantes (empty)
      * @param mcb (empty)
      */
    void calcRelevants(list<Prototype>& prototypes,
                      list<Cycle*>& relevants, list<Cycle*>& mcb);

    /**
      * Computes the interchangeability Classes of all cycles in
      * <code>relevants</code> and stores the number of the interchange-
      * ability class in Cycle::interclass
      * see Gleiss et al. (1999) pp10.
      *
      * @param relevantes (for this biconnected component)
      * @param index of the next xchg-class
      */
    void calcXchgClasses(list<Cycle*>& relevants,
                          unsigned long& index);

    /**
      * Computes the edge-short and unique edge-short cycles
      * from the list of <code>relevants</code>.
      * Sets the flag short / unique short in the list of all cycles.
      *
      * @param relevants
      */
    void calcShortests(list<Cycle*>& relevants);

    /**
      * Computes the intersection of all minimal cycle bases from
      * the list of <code>relevants</code> and a <code>mcb</code>.
      * Sets the flag short / unique short in the list of all cycles.
      * see Gleiss, Stadler (1999) p 7 Table 1
      */
    void calcEssentials(list<Cycle*>& relevants, list<Cycle*>& mcb);

    /**
      * Adds the Node with id <code>node</code> to <code>liste</code>.
      *
      * @param list<Node>& to add the node
      * @param a valid node id
      */
    void int2Nodes(list<Node>& liste, int node) const;

    /**
      * Adds the Nodes from <code>path</code> to <code>liste</code>
      * in the same orientation if
      * <code>right</code> is true, else in reverse order.
      *
      * @param list<Node>& to add the node
      * @param a path with nodes with valid ids
      * @param right
      */
    void int2Nodes(list<Node>& liste, const Path& path,bool right) const;

    /**
      * Initialize EdgeIdMap.
      * Calculates biconnected components, if not already calculated.
      */
    void init();

    /**
      * Portion out U-nodes to the components.
      */
    void unodes2Comp();

    /**
      * Initalize edgeMap:
      * Each each is paired with an unique unsigned int.
      */
    void initEdgeIdMap();

    /**
      * Sets the index over all cycles and U-elements.
      * If <code>isUBasis</code> is false,
      * the cycles are moved to <code>cycleVec</code>
      * elso to <code>uElemVec</code>.
      *
      * @param isUBasis
      */
    void setCycleIndex(bool isUBasis = false);

    /**
      * Makes a new list<BitVector> and fills it with the cycles from
      * <code>lessM</code> and <code>equals</code>.
      *
      * @param lessM cycles contained in an MCB with less length
      * @param equals relevant cycles of the same length
      * @return matrix of cycles
      */
    list<BitVector>* makeCycleMatrix(const list<Cycle*> lessM,
                                     const vector<Cycle*> equals) const;


    /**
      * Does the coloring for the xchg-classes
      *   (see Algorithm in Gleiss et al. (2000))
      * and assigns the xchgIndex to the relevant cycles
      *
      * @param matrix: the reduced row echelon form of the cycle matrix
      * @param the relevant cycles of same length
      * @param next xchg Index
      * @param a BitVector to unmask the edges from the cycles of less length
      */
    void colorCycleMatrix(const list<BitVector>& matrix, vector<Cycle*>& equals,
                    unsigned long& xchgIndex, const BitVector& mask) const;


    /**
      * Copies all cycle-prototypes for cycleVec into <code>prototypes</code>.
      */
    void copyCyclePrototypes2UElem(list<Prototype>& prototypes);

    /**
      * Copies all cycle-relevants
      */
    void copyCycleRelevants2UElem();

  protected:
    //------------------------------------------------------ Datas
    /// list of cycles
    vector<Cycle*> cycleVec;

    /// list of cycles and u-paths
    vector<Cycle*> uElemVec;

    /// list of u-nodes
    vector<Node> unodes;

    /// number of connected components
    unsigned components;

    /// number of biconnected components
    unsigned bicomponents;

    /// list of biconnected components
    vector<Component* > biComponents;

    /// list of connected components
    vector<Component* > conComponents;

    /// list of u-nodes, portion out of components
    vector<list<Node> > unodesComp;

  private:
    /// list of cycles - during calculations
    list<Cycle*> cycles;

    /// list of digraphs
    list<Graph*> digraphs;

    /// list of digraphs for unodes
    list<Graph*> unodeDigraphs;

    /// status of calculation
    CalcStat calcStat;

    /// map id (unsigned) <-> Edge
    EdgeIdMap  edgeMap;

    /// for debugging -> writes: [node.id] distance marked "path:" path
    void writeNodeMap(const NodePathMap&, ostream&) const;



}; // class CycleGraph

} // namespace graphlib

#endif // CYCLEGRAPH_H

/* end of file */
