/***************************************************************************
                          eigenvector_eigenvalue.h  -  description
                             -------------------
    begin                : Wed Mar 14 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef EIGENVECTOR_EIGENVALUE_H
#define EIGENVECTOR_EIGENVALUE_H

#include "basisgraph.h"
#include "eigenpair.h"
#include "matrix.h"

namespace graphlib {


/**
  * @short eigenvalues and eigenvectors of a graph
  *
  * Graphs with only one node, are not allowed. (Bug in ccmath,
  * in function housev()).
  *
  */

class Eigenvector_Eigenvalue
{
  	//------------------------------------------------------ friend functions
    /**
      * Output-operator.
      *
      * @param os the output-stream
      * @param eV list of EigenPair
      * @return the output-stream
      */
    friend ostream& operator<<(ostream& os,const Eigenvector_Eigenvalue& eV);

  public:
  	//------------------------------------------------------ Con-/Destructors
    /**
      * Constructor: Creates a list of EigenPairs, holding eigenvectors
      * and eigenvalue of the matrix <code>l</code>.
      *
      *
      * @param <code>l</code> the matrix
      */
  	Eigenvector_Eigenvalue(const Matrix* l);
  	
  	/**
  	  * Destructor: Deletes all eigenpairs.
  	  */
  	~Eigenvector_Eigenvalue();
   	 	
  	//------------------------------------------------------ Output
  	
  	/**
  	  * Writes the eigenvalues and eigenvectors to <code>out</code>.
  	  * Format:
  	  * first line : space    all eigenvalues
  	  * other lines: node.id  eigenvectors
  	  *
  	  * @param map Node<->unsigned
  	  * @param ostream (default stdout)
  	  */
  	void write(const NodeNumberMap& nodes, ostream& out=cout) const;

  	/**
  	  * Writes the eigenvalues and eigenvectors to <code>out</code>.
  	  * Format: mathematic
  	  * eval = { .. , .. };
  	  * evev = { { .. , .. } , { .. , .. } };
  	  *
  	  * eval and evec: format of doubles: 1.xxxxxxxxxxxxxxxx*^exponent
  	  *
  	  * @param ostream (default stdout)
  	  */
  	void writeMathematica(ostream& out=cout);
  	
  	//------------------------------------------------------ Information
  	/**
  	  * Returns the list of EigenPairs.
  	  */
  	list<EigenPair*> eigenPairs() { return eigenpairs; }
  	
 	  /**
  	  * Returns the begin of list of EigenPairs.
  	  */
  	constEigenPairIter eigenPairsBegin() const { return eigenpairs.begin(); }
  	
 	  /**
  	  * Returns the end of list of EigenPairs.
  	  */
  	constEigenPairIter eigenPairsEnd() const { return eigenpairs.end(); }
  	
  	/**
  	  * Returns smallest EigenValue.
  	  */
  	double minEigenValue() const;
  	
  	/**
  	  * Returns greatest EigenValue.
  	  */
  	double maxEigenValue() const;
  	
  	/**
  	  * Returns lowest value of EigenVectors.
  	  */
  	double minEigenVector() const { return minimal; }
  	
  	/**
  	  * Returns highest value of EigenVectors.
  	  */
  	double maxEigenVector() const { return maximal; }
  	  	  	  	
  private:
  	//------------------------------------------------------ Data
    /// list of eigenpairs: eigenvalue + eigenvector
    list<EigenPair*>           eigenpairs;

    /// minimal value of eigenvectors
    double minimal;
    /// maximal value of eigenvectors
    double maximal;

  	//------------------------------------------------------ Utilities
  	/**
  	  * Calculates Eigenvalues and Eigenvectors
  	  *
  	  * @param matrix
  	  */
    void makeEigen(const Matrix&); 	
  	
  	
}; // class Eigenvector_Eigenvalue

} // namespace graphlib
#endif  //EIGENVECTOR_EIGENVALUE_H

/* end of file */
