/***************************************************************************
                          gml_scanner.h  -  description
                             -------------------
    begin                :
    copyright            : (C) 1997 by Michael Himsolt
    email                : himsolt@fmi.uni-passau.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef GML_SCANNER_H
#define GML_SCANNER_H

#include <cstdio>
#include <string>

namespace graphlib {
/*
 * start-size of buffers for reading strings. If too small it will be enlarged
 * dynamically
 */

#define INITIAL_SIZE 1024

/**
  * Possible value types while parsing a GML file
  */
typedef enum { GML_KEY,
               GML_INT,
               GML_DOUBLE,
               GML_STRING,
               GML_L_BRACKET,
               GML_R_BRACKET,
               GML_END,
               GML_LIST,
               GML_ERROR
             } GML_value;

/**
  * Possible errors while parsing a GML file.
  * <ul>
  *   <li>
  *     <code>GML_EDGE_NODE_UNKNOWN</code>: source or target of the edge is
  *     unknown.
  *   </li>
  *   <li>
  *     <code>GML_EDGE_TWICE</code>: source or target of the edge is
  *     unknown.
  *   </li>
  *   <li>
  *     <code>GML_FILE_NOT_FOUND</code>: error while opening the GML file.
  *   </li>
  *   <li>
  *     <code>GML_NODE_ID_DOUBLE</code>: a node id occures twice.
  *   </li>
  *   <li>
  *     <code>GML_NODE_NO_ID</code>: a node without id.
  *   </li>
  *   <li>
  *     <code>GML_NO_GRAPH_FOUND</code> no keyword graph in the file
  *   </li>
  *   <li>
  *     <code>GML_OK</code>: no error :-))
  *   </li>
  *   <li>
  *     <code>GML_OPEN_BRACKET</code>: too many opening brackets <code>[</code>
  *   </li>
  *   <li>
  *     <code>GML_PREMATURE_EOF</code>: an EOF occured, where it was not expected,
  *       i.e. while parsing a string 
  *   </li>
  *   <li>
  *     <code>GML_SYNTAX</code>: the file is not a valid GML file, i.e. mismatch
  *       in the key-value pairs.
  *   </li>
  *   <li>
  *     <code>GML_TOO_MANY_BRACKETS</code>: a mismatch of brackets was detected,
  *       i.e. too many closing brackets <code>]</code>
  *   </li>
  *   <li>
  *     <code>GML_TOO_MANY_DIGITS</code> the number of digits a floating point or
  *       integer value can hold is limited to 1024
  *   </li>
  *   <li>
  *     <code>GML_UNEXPECTED</code>: a character occured, where i makes no sense,
  *       e.g. non-numerical characters in numbers or keys beginning with numbers 
  *   </li>
  * </ul>
  *
  */
typedef enum  { GML_OK=0,
                GML_UNEXPECTED,
                GML_SYNTAX,
                GML_PREMATURE_EOF,
                GML_TOO_MANY_DIGITS,
                GML_OPEN_BRACKET,
                GML_TOO_MANY_BRACKETS,
                GML_FILE_NOT_FOUND,
                GML_NODE_ID_DOUBLE,
                GML_NO_GRAPH_FOUND,
                GML_NODE_NO_ID,
                GML_EDGE_NODE_UNKNOWN,
                GML_EDGE_TWICE
              } GML_error_value;


/**
  * @short REason and position of an error in a GML file.
  *
  * When an error accurs while parsing the structure of a GML file
  * <code>GML_error</code> is used to return the type and position of 
  * the error detected. Position is specified by <code>line</code> and
  * <code>column</code>, but might be somewhat imprecise. However at 
  * least the line number hsould not differ too much from the real
  * position.
  *
  * @see graph#loadGML 
  */
struct GML_error
{
    GML_error_value err_num;
    int line;
    int column;
};


/**
  * @internal
  */
union GML_tok_val
{
    long integer;
    double floating;
    char* str;
    struct GML_error err;
};


/**
  * @internal
  */
struct GML_token
{ 
    GML_value kind;
    union GML_tok_val value;
};

/*
 * global variables
 */

extern unsigned int GML_line;
extern unsigned int GML_column;
extern string GML_table[];

/**
  * if you are interested in the position where an error occured it is a good
  * idea to set GML_line and GML_column back. 
  * This is what GML_init does.
  */
 
void GML_init ();

/**
  * returns the error message of an error
  */
string GML_error2string(struct GML_error);

/**
  * returns the next token in file. If an error occured it will be stored in 
  * GML_token.
  */

struct GML_token GML_scanner (FILE*);

} // namespace graphlib

#endif  // GML_SCANNER_H

/* end of file */



