/***************************************************************************
                          graph.cpp  -  description
                             -------------------
    begin                : Thu Mar 1 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "graph.h"

#include "nodedata.h"
#include "nodeid.h"

#include "edgedata.h"
#include "edgeid.h"

#include "matrix.h"

#include "utilities.h"
#include "trace.h"
#include "logfile.h"
#include "graphlib_error.h"
using namespace graphlibError;

#include <algorithm>
#include <strstream>
#include <cmath>

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
Graph::Graph() : BasisGraph()
{
  init();
}

Graph::Graph(const Node& node) : BasisGraph()
{
  init();
  copyNode(node);
}

Graph::~Graph()
{
  clear();
}

//---------------------------------------------------------------------------
// Operator
//---------------------------------------------------------------------------
Graph& Graph::operator=(const Graph& g)
{
  copy(g);
  return *this;
}

Graph& Graph::operator=(const Matrix& am)
{
  unsigned size=am.size();
  const double *matrix = am.matrix();

  if (!matrix)
    THROW(EmptyMatrixError,"Graph::Graph(const Matrix&)")

  for (unsigned i=0; i+1!=size; i++)
  {
    // new node with id=i+1
    newNode();
    for (unsigned j=i; j!=size; j++)
    {
      // 1 -> edge(i+1,j+1)
      if (matrix[i*size+j] == 1)
        newEdge(i+1,j+1);
     }
  }
  return *this;
}

//---------------------------------------------------------------------------
// Input / Output
//---------------------------------------------------------------------------
GML_error Graph::load(const string& fname)
{
  GML_stat stat;
  stat.key_list = NULL;
  GML_pair* keyList;
  GML_pair* origList;

  // initialize stat
  stat.err.line = stat.err.column = 0;

  // open file
  FILE* file = fopen(fname.c_str(), "r");

  if (!file)
  {
    stat.err.err_num = GML_FILE_NOT_FOUND;
    return stat.err;
  }

  // initialize GML parser, parse file and close it
  GML_init();
  keyList = GML_parser(file, &stat, 0);
  fclose(file);

  // print keyList
  #ifdef GRAPHLIB_DEBUG
    #if GRAPHLIB_DEBUG>4
    if (logfile.isInit())
      GML_printList(keyList,0,logfile.os());
    #endif
  #endif

  // error occured
  if (stat.err.err_num != GML_OK)
  {
    GML_free_list(keyList, stat.key_list);
    return stat.err;
  }

  //
  // valid GML file -> build the graph
  //
  clear();
  origList = keyList;

  //
  // get the first entry with key "graph" in the list and move it to the
  // beginning
  //
  GML_pair* prev = 0;

  while (keyList)
  {
    if (!strcmp("graph",keyList->key))
      break;

    prev = keyList;
    keyList = keyList->next;
  }

  if (prev)
  {
    prev->next = keyList->next;
    keyList->next = origList;
    origList = keyList;
  }

  // keys of the graph (directed, name, node, edge, ...)
  keyList = keyList->value.list;

  GML_pair* lastNode = keyList;

  GML_pair* tmpList;
  GML_pair* tmpPrev;
  GML_pair* nodeEntries = 0;
  GML_pair* edgeEntries = 0;

  prev = 0;
  bool targetFound, sourceFound;

  //
  // Node and edge keys maybe in arbitrary order -> sort them
  // all nodes before all edges
  //
  while (keyList)
  {
    if (!strcmp(keyList->key, "node"))
    {
      // search the list associated with this node for the id
      // move id to the front of the list
      tmpList = keyList->value.list;
      tmpPrev = 0;

      while (tmpList)
      {
        if (!strcmp(tmpList->key, "id"))
        {
          if (tmpList->kind != GML_INT)
            THROW(runtime_error,
                "Graph::load(const string&) node id not an integer!")

          if (tmpPrev == 0)
            break;
          else
          {
            tmpPrev->next = tmpList->next;
            tmpList->next = keyList->value.list;
            keyList->value.list = tmpList;
            break;
          }
        }
        tmpPrev = tmpList;
        tmpList = tmpList->next;
      }

      if (prev)
      {
        prev->next = keyList->next;
        keyList->next = nodeEntries;
        nodeEntries = keyList;
        keyList = prev->next;
        continue;
      }
      else
      {
        origList->value.list = keyList->next;
        keyList->next = nodeEntries;
        nodeEntries = keyList;
        keyList = origList->value.list;
        continue;
      }
    }
    else if (!strcmp(keyList->key, "edge"))
    {
      // search for source and target
      // move them to front of list
      tmpList = keyList->value.list;
      tmpPrev = 0;
      sourceFound = false;
      targetFound = false;

      while (tmpList)
      {
        if (!strcmp(tmpList->key, "source"))
        {
          if (tmpList->kind != GML_INT)
            THROW(runtime_error,
                "Graph::load(const string&) edge source not an integer!")
          sourceFound = true;

          if (tmpPrev)
          {
            tmpPrev->next = tmpList->next;
            tmpList->next = keyList->value.list;
            keyList->value.list = tmpList;
            tmpList = tmpPrev;
            if (targetFound)
              break;
          }

        }
        else if (!strcmp(tmpList->key, "target"))
        {
          if (tmpList->kind != GML_INT)
            THROW(runtime_error,
                "Graph::load(const string&) edge target not an integer!")
          targetFound = true;

          if (tmpPrev)
          {
            tmpPrev->next = tmpList->next;

            if (sourceFound)
            {
              tmpList->next = keyList->value.list->next;
              keyList->value.list->next = tmpList;
              break;
            }
            else
            {
              tmpList->next = keyList->value.list;
              keyList->value.list->next = tmpList;
              tmpList = tmpPrev;
            }
          }
        }
        tmpPrev = tmpList;
        tmpList = tmpList->next;
      }

      if (prev)
      {
        prev->next = keyList->next;
        keyList->next = edgeEntries;
        edgeEntries = keyList;
        keyList = prev->next;
        continue;
      }
      else
      {
        origList->value.list = keyList->next;
        keyList->next = edgeEntries;
        edgeEntries = keyList;
        keyList = origList->value.list;
        continue;
      }
    }
    else if (!strcmp(keyList->key, "directed"))
    {
      // ignore at the moment
      // remove from key list
      if (prev)
      {
        prev->next = keyList->next;
        free (keyList);
        keyList = prev;
      }
      else
      {
        origList->value.list = keyList->next;
        lastNode = keyList->next;
        free (keyList);
        keyList = origList->value.list;
        continue;
      }
    }
    else if (!strcmp(keyList->key, "name"))
    {
      // name of the graph
      // remove from key list
      name_ = keyList->value.str;
      if (prev)
      {
        prev->next = keyList->next;
        free (keyList);
        keyList = prev;
      }
      else
      {
        origList->value.list = keyList->next;
        lastNode = keyList->next;
        free (keyList);
        keyList = origList->value.list;
        continue;
      }
    }
    prev = keyList;
    keyList = keyList->next;
  }

  //
  // make the graph described in the list
  //

  // nodes
  loadNodes(nodeEntries, &stat);
  if (stat.err.err_num != GML_OK)
    return stat.err;

  // edges
  loadEdges(edgeEntries, &stat);
  if (stat.err.err_num != GML_OK)
    return stat.err;

  GML_free_list(origList, stat.key_list);
  GML_free_list(nodeEntries,0);
  GML_free_list(edgeEntries,0);
  stat.err.err_num = GML_OK;
  return stat.err;
}

GML_error Graph::loadNodes(GML_pair* nodeEntries, GML_stat* stat)
{
  GML_pair* node;
  GML_pair* graphics;
  double tmp;
  bool nodeHasId=false;

  while (nodeEntries)
  {
    node = nodeEntries->value.list;
    Node n;
    n.data_ = new NodeData;

    while (node)
    {
      if (!strcmp(node->key, "id"))
      {
        if (getFreeNodeNumber(node->value.integer) != true)
        {
          stat->err.err_num = GML_NODE_ID_DOUBLE;
          stat->err.line = nodeEntries->lineNr;
          return stat->err;
        }
        n.data_->id = node->value.integer;
        nodeHasId = true;
      }
      else if (!strcmp(node->key, "label") || !strcmp(node->key, "parameter"))
        n.data_->label = node->value.str;

      else if (!strcmp(node->key, "graphics"))
      {
        graphics = node->value.list;

        while (graphics)
        {
          tmp = graphics->value.floating;
          if (!strcmp(graphics->key, "x"))
          {
            n.data_->coord.x = tmp;
            if (maxCoords.x < tmp) maxCoords.x=tmp;
            else if (minCoords.x > tmp) minCoords.x=tmp;
          }
          else if (!strcmp(graphics->key, "y"))
          {
            n.data_->coord.y = tmp;
            if (maxCoords.y < tmp) maxCoords.y=tmp;
            else if (minCoords.y > tmp) minCoords.y=tmp;
          }
          else if (!strcmp(graphics->key, "z"))
          {
            n.data_->coord.z = tmp;
            if (maxCoords.z < tmp) maxCoords.z=tmp;
            else if (minCoords.z > tmp) minCoords.z=tmp;
          }
          graphics = graphics->next;
        }
      }
     node = node->next;
    }
    //node without id
    if (nodeHasId == false)
    {
      stat->err.err_num = GML_NODE_NO_ID;
      stat->err.line = nodeEntries->lineNr;
      delete n.data_;
      return stat->err;
    }

    // add new node to graph
    n.data_->pos = nodes.insert(findGreaterNode(n.id()),n);

    // first node
    if (nodes.size() == 1)
      minCoords = maxCoords = n.coord();

    // next node
    nodeEntries = nodeEntries->next;
  }
  return stat->err;
}

GML_error Graph::loadEdges(GML_pair* edgeEntries, GML_stat* stat)
{
  GML_pair* edge;
  int source, target;
  string label;

  while (edgeEntries)
  {
    edge = edgeEntries->value.list;
    source = target = 0;
    label = "";

    while (edge)
    {
      if (!strcmp(edge->key, "source"))
        source = edge->value.integer;

      else if (!strcmp(edge->key, "target"))
        target = edge->value.integer;

      else if (!strcmp(edge->key, "label") || !strcmp(edge->key, "parameter"))
        label = edge->value.str;

      else
        TRACE("edge: " + Ttos(source) + "->" + Ttos(target) + " unexpected keys:" +
        edge->key)

      edge = edge->next;
    }
    try
    {
      newEdge(source, target, label);
    }
    catch (DoubleError &e)
    {
      THROW(runtime_error,Ttos(edgeEntries->lineNr)+e.what())
      stat->err.err_num = GML_EDGE_NODE_UNKNOWN;
      stat->err.line = edgeEntries->lineNr;
      return stat->err;
    }
    catch (logic_error &e)
    {
      stat->err.err_num = GML_EDGE_NODE_UNKNOWN;
      stat->err.line = edgeEntries->lineNr;
      return stat->err;
    }
    edgeEntries = edgeEntries->next;
  }
  return stat->err;
}


//---------------------------------------------------------------------------
// Information
//---------------------------------------------------------------------------
string Graph::name() const
{
  return name_;
}
void Graph::name(string name)
{
  name_ = name;
}
void Graph::newMinMaxCoord(double x, double y)
{
  if (maxCoords.x < x)
    maxCoords.x = x;
  if (minCoords.x > x)
    minCoords.x = x;

  if (maxCoords.y < y)
    maxCoords.y = y;
  if (minCoords.y > y)
    minCoords.y = y;
}
void Graph::resetMinMaxCoord()
{
  minCoords = +HUGE_VAL; // inf
  maxCoords = -HUGE_VAL; // -inf
}

//---------------------------------------------------------------------------
// Search
//---------------------------------------------------------------------------
vector<Path> Graph::vectorPaths(int x, int r, Path& currentPath)
{
  vector<Path> result, help;
  Path myCurrentPath;
  constNodeIter z;

  // assign
  myCurrentPath=currentPath;
  // += {x}
  myCurrentPath.push_back(x);

  // all pathes between x and r found -> END
  if (x==r)
  {
    // add current path of x
    result.push_back(myCurrentPath);
    return result;
  }

  else
  {
    // empty result
    result.clear();

    // forall nodes
    for(z=nodes.begin();z!=nodes.end();z++)
    {
      // exits edge = {x,z} ?
      if (z->isTarget(x))
      {
        // found path
        help=(vectorPaths(z->id(),r,myCurrentPath));
        // add found path to result
        result.insert(result.end(),help.begin(),help.end());
      }
    }
    return result;
  }
  return result;
}


//---------------------------------------------------------------------------
// Creation
//---------------------------------------------------------------------------
NodeIter Graph::newNode()
{
  // create node
  Node n;
  n.data_ = new NodeData;
  n.data_->id = getFreeNodeNumber();
  // insert befor first greater node
  n.data_->pos=nodes.insert(findGreaterNode(n.id()),n);

  return n.data_->pos;
}

EdgeIter Graph::newEdge(const Node& s, const Node& t, string label)
{
  string error="Graph::newEdge(const Node&, const Node&, string): ";

  if (!s.data_ || !t.data_)
    THROW(EmptyNodeError,error+"source or target has no data.")

  if (findEdge(s.id(),t.id()) != edges.end())
    THROW(DoubleError,error+"edge already exists:"+Ttos(s.id())+" "+Ttos(t.id()))


  // create edge
  Edge e;
  e.data_ = new EdgeData;

  // set id and source and target
  e.data_->nodes.push_back(s);
  e.data_->nodes.push_back(t);
  e.data_->label=label;

  // add edge to graph
  e.data_->pos=edges.insert(edges.end(),e);

  // set adj_pos
  list<Edge>& sourceAdj = s.data_->edges;
  list<Edge>& targetAdj = t.data_->edges;

  e.data_->adj_pos.push_front(sourceAdj.insert(sourceAdj.end(),e));
  e.data_->adj_pos.push_back(targetAdj.insert(targetAdj.begin(),e));

  s.data_->degree++;
  t.data_->degree++;

  return e.data_->pos;
}

EdgeIter Graph::newEdge(int source, int target, string label)
{
  string error="Graph::newEdge(int,int,string): ";
  ostrstream err;
  err << "Edge (" << source << "," << target << ") ";
  error += err.str();
  err.rdbuf()->freeze(false);

  constNodeIter s,t;
  if ((s=findNode(source)) == nodes.end() ||
      (t=findNode(target)) == nodes.end())
    THROW(logic_error,error+"source or target not found")

  return newEdge(*s,*t,label);
}

Graph* Graph::makeSubgraph(constNodeIter begin, constNodeIter end, bool induced)
{
  // removes all nodes and edges
  if (!nodes.empty())
    clear();

  constNodeIter it = begin;

  while (it != end)
  {
    // copy node
    copyNode(*it);

    if (induced)
    {
      // copy adjacent edges
      constEdgeIter edge;
      for (edge = it->adjBegin(); edge!=it->adjEnd(); edge++)
      {
        if (findNode(it->opposite(*edge).id()) != nodes.end())
          copyEdge(*edge);
      }
    }
    it++;
  }
  return this;
}

Graph* Graph::makeSubgraph(cVEdgeIter begin, cVEdgeIter end)
{
  // removes all nodes and edges
  if (!nodes.empty())
    clear();

  cVEdgeIter it = begin;

  while (it != end)
  {
    // copy source and target of edge
    try
    {
      copyNode(it->source());
    }
    catch (DoubleError)  { ; } // node already exists

    try
    {
      copyNode(it->target());

    }
    catch (DoubleError) { ; } // node already exists

    copyEdge(*it);
    it++;
  }
  return this;
}

//---------------------------------------------------------------------------
// Deletion
//---------------------------------------------------------------------------
void Graph::delNode(Node& n)
{
  if (!n.data_ )
    THROW(logic_error,"Graph::delNode(const Node&): node has no data.")
  int freeNumber = n.data_->id;

  // delete incident edges
  while (n.data_->edges.begin() != n.data_->edges.end())
  {
    delEdge(*n.data_->edges.begin());
  }

  // delete node from the graph
  nodes.erase(n.data_->pos);
  setFreeNodeNumber(freeNumber);
  delete n.data_; n.data_=0;
}

void Graph::delAllNodes()
{
  clear();
}

void Graph::delEdge(Edge& e)
{
  if (!e.data_)
    THROW(logic_error,"Graph::delEdge(const Edge&): edge has no data.")

  // remove edge form adjacency list
  e.removeFrom();

  // delete edge from the graph
  edges.erase(e.data_->pos);
  delete e.data_; e.data_=0;
}

void Graph::delAllEdges()
{
  NodeIter it = nodes.begin();
  NodeIter end = nodes.end();

  while (it != end)
    (it++)->data()->edges.clear();

  delList(edges);
}

void Graph::clear()
{
  if (!edges.empty())
    delList(edges);
  if (!nodes.empty())
    delList(nodes);

  name_ = ""; number_ = 0;
  resetMinMaxCoord();
  setFreeNodeNumber();
}

//---------------------------------------------------------------------------
// Utilities for copy() - CopyConstructor and AssignementOperator
//---------------------------------------------------------------------------
void Graph::init()
{
  name_ = "";
  setFreeNodeNumber();
  resetMinMaxCoord();
}

void Graph::copy(const Graph& g)
{
  // clear all nodes and edges
  clear();

  // copy all nodes
  copyList(g.nodesBegin(),g.nodesEnd());

  // copy all edges
  copyList(g.edgesBegin(),g.edgesEnd());

  // copy other attributes
  name_ = g.name_;
  number_ = g.number_;
  minCoords   = g.minCoords;
  maxCoords   = g.maxCoords;
  freeNodeNumbers = g.freeNodeNumbers;
}

void Graph::copyList(constNodeIter it, constNodeIter end)
{
  while(it != end)
  {
    copyNode(*(it++));
  }
}

void Graph::copyNode(const Node& old)
{
  if (findNode(old.id()) != nodes.end())
  {
    ostrstream err;
    err << "Graph::copyNode(const Node&): id (" << old.id()
        << ") already exists." << ends;
    string tmp(err.str());
    err.rdbuf()->freeze(false);
    THROW(DoubleError,tmp)
  }

  // create node
  Node n;
  n.data_ = new NodeData;

  // set id, label, ...
  n.data_->id     = old.id();
  n.data_->label  = old.label();
  n.data_->coord  = old.coord();
  n.data_->degree = 0;

  // add to graph
  n.data_->pos=nodes.insert(nodes.end(),n);
}

void Graph::copyList(constEdgeIter it, constEdgeIter end)
{
  while(it != end)
  {
    copyEdge(*(it++));
  }
}

void Graph::copyEdge(const Edge& old)
{
  newEdge(old.source().id(),old.target().id(),old.label());
}

//---------------------------------------------------------------------------
// Utilities for clear() - Destructor
//---------------------------------------------------------------------------
void Graph::delList(list<Node>& l)
{
  constNodeIter it = l.begin();
  constNodeIter end = l.end();

  while(it != end)
  {
    delete (it++)->data();
  }
  l.clear();
}

void Graph::delList(list<Edge>& l)
{
  constEdgeIter it = l.begin();
  constEdgeIter end = l.end();

  while(it != end)
  {
    delete (it++)->data();
  }
  l.clear();
}

//---------------------------------------------------------------------------
// Utilities for creating and deleting nodes/edges
//---------------------------------------------------------------------------
bool Graph::getFreeNodeNumber(int number)
{
  if (number == INFINITY)
    THROW(runtime_error,"Graph::newNode(int): number to big!")

  list<int>::iterator nextNumber;
  list<int>::iterator it;
  int tmp;

  // negativ number
  if (number < 0)
  {
    // the next free negativ number
    nextNumber = (++freeNodeNumbers.begin());
    tmp = *nextNumber;

    // number is smaller or equal then the next free one
    if (number <= tmp)
    {
      // new smallest negativ number
      *nextNumber = number-1;
      // add all numbers between number and the old smallest number
      for (int i=tmp; i!=number; i--)
        freeNodeNumbers.push_back(i);

      return true;
    }
    // number has to be in the "gab" list
    else
    {
      // search for the number in the "gab" list
      it = find(nextNumber,freeNodeNumbers.end(),number);

      // number is in the list
      if (it != freeNodeNumbers.end())
      {
        freeNodeNumbers.erase(it);
        return true;
      }
      // number is not in the list == already exists
      else
        return false;
   }
  }

  // negativ number
  else
  {
    // the next free negativ number
    nextNumber = freeNodeNumbers.begin();
    tmp = *nextNumber;

    // number is bigger or equal then the next free one
    if (number >= tmp)
    {
      // new biggest positive number
      *nextNumber = number+1;
      // add all numbers between number and the old biggest number
      for (int i=tmp; i!=number; i++)
        freeNodeNumbers.push_back(i);

      return true;
    }
    // number has to be in the "gab" list
    else
    {
      // search for the number in the list
      it = find(nextNumber,freeNodeNumbers.end(),number);

      // number is in the list
      if (it != freeNodeNumbers.end())
      {
        freeNodeNumbers.erase(it);
        return true;
      }

      // number is not in the list
      else
         return false;

    }
  }
}

int Graph::getFreeNodeNumber()
{
  int tmp;
  // if no gaps -> always assign positive node ids
  if (freeNodeNumbers.size() == 2)
    tmp = (*freeNodeNumbers.begin())++;

  else
  {
    tmp = freeNodeNumbers.back();
    freeNodeNumbers.pop_back();
  }
  return tmp;
}

void Graph::setFreeNodeNumber()
{
  // first element
  freeNodeNumbers.clear();
  // for the positive numbers
  freeNodeNumbers.push_back(1);
  // for the negativ numbers
  freeNodeNumbers.push_back(0);
}

void Graph::setFreeNodeNumber(int number)
{
  // node number is free -> add
  freeNodeNumbers.push_back(number);
}

} // namespace graphlib

/* end of file */
