/***************************************************************************
                          matrix.cpp  -  description
                             -------------------
    begin                : Wed Mar 14 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "matrix.h"
#include "trace.h"
#include "basisgraph.h"

#include <algorithm>

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
Matrix::Matrix(unsigned asize)
      :  matrix_(0),size_(asize)
{
  matrix_ = new double[size_*size_];
  fill(matrix_,matrix_+size_*size_,0);
}

Matrix::Matrix(unsigned asize, const double* am)
      :  matrix_(0),size_(asize)
{
  if (am == 0)
    THROW(EmptyMatrixError,"Matrix::Matrix(unsigned, const double*)")

  matrix_ = new double[size_*size_];
  fill(matrix_,matrix_+size_*size_,0);

  for (unsigned int i=0; i!=size_*size_; i++)
    matrix_[i] = am[i];
}

Matrix::Matrix(const Matrix& am) throw(EmptyMatrixError)
      :  matrix_(0),size_(am.size_)
{
  copy(am);
}

Matrix::~Matrix()
{
  if (matrix_)
    delete[] matrix_;

  matrix_ = 0;
}

//---------------------------------------------------------------------------
// Operator
//---------------------------------------------------------------------------
Matrix& Matrix::operator=(const Matrix& am) throw(EmptyMatrixError)
{
  if (this == &am) return *this;
  copy(am);
  return *this;
}

Matrix Matrix::operator+(const Matrix& am) const
    throw(EmptyMatrixError,SizeError)
{
  if (! am.matrix_)
    THROW(EmptyMatrixError,"Matrix::operator+(const Matrix&) const")

  if (am.size_ != size_)
    THROW(SizeError,"Matrix::operator+(const Matrix&) const")

  Matrix tmp(size_);

  for (unsigned i=0; i!=size_*size_; i++)
    tmp.matrix_[i] = matrix_[i] + am.matrix_[i];

  return tmp;
}

Matrix Matrix::operator-(const Matrix& am) const
    throw(EmptyMatrixError,SizeError)
{
  if (! am.matrix_)
    THROW(EmptyMatrixError,"Matrix::operator-(const Matrix&) const")

  if (am.size_ != size_)
    THROW(SizeError,"Matrix::operator-(const Matrix&) const")

  Matrix tmp(size_);

  for (unsigned i=0; i!=size_*size_; i++)
    tmp.matrix_[i] = matrix_[i] - am.matrix_[i];

  return tmp;
}

Matrix& Matrix::operator+=(const Matrix& am)
    throw(EmptyMatrixError,SizeError)
{
  if (! am.matrix_)
    THROW(EmptyMatrixError,"Matrix::operator+=(const Matrix&)")

  if (am.size_ != size_)
    THROW(SizeError,"Matrix::operator+=(const Matrix&) const")

  for (unsigned i=0; i!=size_*size_; i++)
    matrix_[i] += am.matrix_[i];

  return *this;
}

Matrix& Matrix::operator-=(const Matrix& am)
    throw(EmptyMatrixError,SizeError)
{
  if (! am.matrix_)
    THROW(EmptyMatrixError,"Matrix::operator-=(const Matrix&)")

  if (am.size_ != size_)
    THROW(SizeError,"Matrix::operator-=(const Matrix&) const")

  for (unsigned i=0; i!=size_*size_; i++)
    matrix_[i] -= am.matrix_[i];

  return *this;
}

//---------------------------------------------------------------------------
// In-/Output
//---------------------------------------------------------------------------
ostream& operator<<(ostream& os,const Matrix& l)
{
  for (unsigned i=0; i!=l.size_; i++)
  {
    for (unsigned j=0; j!=l.size_; j++)
      os << (int)l.matrix_[l.size_*i+j] << "   ";
    os << endl;
  }

  return os;
}

//void Matrix::write(const NodeNumberMap& nodes, ostream& os)
//{
//  NodeNumberMap::const_iterator node=nodes.begin();
//
//  os.precision(4);
//  os.setf(ios::fixed);
//  os.setf(ios::showpos);
//
//
//  for (unsigned i=0; i!=dim; i++,node++)
//  for (unsigned i=0; i!=l.size_; i++)
//  {
//    os << node->first.id() << "\t";
//    for (unsigned j=0; j!=l.size_; j++)
//      os << (int)l.matrix_[l.size_*i+j] << "   ";
//    os << endl;
//  }
//}

void Matrix::writeMathematica(ostream& os, const string& name, bool isDiagonal)
{
  if (!isDiagonal)
  {
    os << name << " = { { ";

    for (unsigned i=0; i!=size_; i++)
    {
      for (unsigned j=0; j!=size_; j++)
      {
        os << (int)matrix_[size_*i+j];

        if (j!=size_-1)
          os << " , ";
      }
      if (i!=size_-1)
        os << " } , { ";
    }
    os << " } }; " << endl << endl;
  }

  // diagonal matrices
  else
  {
    os << name << " = DiagonalMatrix [ { ";

    for (unsigned i=0; i!=size_; i++)
    {
      os << (int)matrix_[size_*i+i];
      if (i!=size_-1)
        os << " , ";
    }
    os << " } ]; " << endl << endl;
  }

}

//---------------------------------------------------------------------------
// Utitlities
//---------------------------------------------------------------------------
void Matrix::copy(const Matrix& am) throw(EmptyMatrixError)
{
  if (!am.matrix_)
    THROW(EmptyMatrixError,"Matrix::copy(const Matrix&)")

  // set size and nodes
  size_ = am.size_;

  // delete old matrix
  if (matrix_) delete[] matrix_;

  matrix_ = new double[size_*size_];

  for (unsigned int i=0; i!=size_*size_; i++)
    matrix_[i] = am.matrix_[i];

}

} // namespace graphlib

/* end of file */
