/***************************************************************************
                          matrix.h  -  description
                             -------------------
    begin                : Wed Mar 14 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MATRIX_H
#define MATRIX_H

#include "graphlib_error.h"
using namespace graphlibError;

namespace graphlib {

class BasisGraph;


/**
  * @short A quadratic matrix of size * size.
  * The rows are following each other: j.colum + i.row == j*size + i
  */

class Matrix
{
    //------------------------------------------------------ friend functions
    /**
      * Output-operator.
      *
      * @param os the output-stream
      * @param the Matrix
      * @return the output-stream
      */
    friend ostream& operator<<(ostream& os, const Matrix& a);

  public:
    //------------------------------------------------------ Con-/Destructors
    /**
      * Constructor. Creates new empty matrix of <code>size</code>.
      *
      * @param size
      */
    Matrix(unsigned size = 0);

    /**
      * Constructor. Creates a new quadratic matrix
      * copying <code>matrix</code>.
      */
    Matrix(unsigned size, const double* matrix);

    /**
      * CopyConstructor. Makes a copy of the original Matrix.
      * Allocates new memory!
      * Throws an EmptyMatrixError, if there is no matrix to copy.
      */
    Matrix(const Matrix&) throw(EmptyMatrixError);

    /**
      * Destructor. Deletes both matrices.
      */
    ~Matrix();

    //------------------------------------------------------ Operators
    /**
      * Assignement operator. Assigns a copy of the original
      * Matrix. Allocates new memory!
      */
    Matrix& operator=(const Matrix&) throw(EmptyMatrixError);

    /**
      * Addition.
      */
    Matrix operator+(const Matrix&) const throw(EmptyMatrixError,SizeError);

    /**
      * Assignement - addition.
      */
    Matrix& operator+=(const Matrix&) throw(EmptyMatrixError,SizeError);

    /**
      * Subtraction.
      */
    Matrix operator-(const Matrix&) const throw(EmptyMatrixError,SizeError);

    /**
      * Assignement - subtraction.
      */
    Matrix& operator-=(const Matrix&) throw(EmptyMatrixError,SizeError);
    //------------------------------------------------------ Output
//  	/**
//  	  * Writes the eigenvalues and eigenvectors to <code>out</code>.
//  	  * Format:
//  	  * node.id  matrix
//  	  *
//  	  * @param map Node<->unsigned
//  	  * @param ostream (default stdout)
//  	  */
//  	void write(const NodeNumberMap& nodes, ostream& out=cout);

    /**
      * Writes the  and degree matrix to <code>out</code>.
      * Format: mathematica
      * <code>name</code> = { { .. , .. } , { .. , .. } };
      * If it is a diagonal matrix - <code>isDiagonal</code> is true -
      * the following output is produced:
      * <code>name</code> = DiagonalMatrix [ { .. , .. } ];
      *
      * @param ostream (default stdout)
      * @param name of the matrix
      * @param is diagonal
      */
    void writeMathematica(ostream&, const string& name="matrix", bool isDiagonal=false);

    //------------------------------------------------------ Information
    /**
      * Returns the size of the matrix.
      */
    unsigned size() const { return size_; }

    /**
      * Returns the size of the matrix.
      */
    const double* matrix() const { return matrix_; }

  private:
    //------------------------------------------------------ datas
    ///  matrix
    double          *matrix_;
    /// number of vertices
    unsigned        size_;

    //------------------------------------------------------ Utilities
    /**
      * Makes a copy of <code>adjaMat</code>.
      *
      * @param adjaMat
      */
    void copy(const Matrix& adjaMat) throw(EmptyMatrixError);


}; // class Matrix

} // namespace graphlib

#endif // MATRIX_H

/* end of file */
