/***************************************************************************
                          randomgraph.cpp  -  description
                             -------------------
    begin                : Wed Mar 28 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "randomgraph.h"

#include "trace.h"
#include "utilities.h"

#include <cmath>

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
RandomGraph::RandomGraph(unsigned nodes, unsigned edges, GraphFlag flag)
       : Graph()
{
  SubGraph* tmp=0;

  //  no nodes
  if (nodes==0 && edges==0)
    return;

  if (edges == nodes*(nodes-1)/2)
    flag = Complete;

  if (flag == Random)
    tmp = makeRandom(nodes,edges);

  else if (flag == Hamiltonian)
    tmp = makeHamilton(nodes,edges);

  else if (flag == Complete)
    tmp = makeComplete(nodes);

  else if (flag == Bipartite)
    tmp = makeBipartite(nodes,edges);

  else
    ;

  if (tmp) delete tmp;
}

RandomGraph::~RandomGraph()
{ ; }

//---------------------------------------------------------------------------
// additional functions
//---------------------------------------------------------------------------
SubGraph* RandomGraph::makeBipartite(unsigned m, unsigned n,
                                      int xm, int ym1, int ym2,
                                      int xn, int yn1, int yn2)
{
  list<Node> firstN, secondN;
  SubGraph* subgraph = new SubGraph(this);

  NodeIter node;
  bool coord;
  int spaceM, spaceN;

  // coordinates
  if (xm==0 && ym1==0 && ym2==0 && xn==0 && ym1==0 && ym2==0)
    coord = false;
  else
  {
    coord = true;
    if (ym1 > ym2)
      swap(ym1,ym2);
    if (yn1 > yn2)
      swap(yn1,yn2);

    spaceM = (m<2) ? 0 : (ym2-ym1)/(m-1);
    spaceN = (n<2) ? 0 : (yn2-yn1)/(n-1);
  }

  // set min and max coord.
  newMinMaxCoord(min(xm,xn),min(min(ym1,ym2),min(yn1,yn2)));
  newMinMaxCoord(max(xm,xn),max(max(ym1,ym2),max(yn1,yn2)));

  // all nodes of first part
  for (unsigned i=1; i!=m+1; i++)
  {
    // new node
    firstN.push_back(*(node=newNode()));
    subgraph->add(*node);

    // coordinates
    if (coord)
      node->coord(Coordinates(xm,ym1+spaceM*(i-1)));
  }

  // all nodes of second part
  for (unsigned i=1; i!=n+1; i++)
  {
    // new node
    secondN.push_back(*(node=newNode()));
    subgraph->add(*node);

    // coordinates
    if (coord)
      node->coord(Coordinates(xn,yn1+spaceN*(i-1)));
  }

  // no edges
  if (m == 0 || n == 0)
    return subgraph;

  constNodeIter it1  = firstN.begin();
  constNodeIter end1 = firstN.end();
  constNodeIter it2  = secondN.begin();
  constNodeIter end2 = secondN.end();

  // edges between all nodes of first to all nodes of second part
  while (it1 != end1)
  {
    while (it2 != end2)
      subgraph->add(*newEdge(*it1, *it2++));

    it1++; it2=secondN.begin();
  }

  return subgraph;
}

SubGraph* RandomGraph::makeComplete(unsigned nodes, int radius, int xM, int yM)
{
  SubGraph* subgraph = new SubGraph(this);

  // no nodes -> nothing to do
  if (nodes == 0)
    return subgraph;

  bool coord=false;
  NodeIter node;

  // set min and max coord.
  newMinMaxCoord(xM-radius,yM-radius);
  newMinMaxCoord(xM+radius,yM+radius);

  if (radius > 0)
    coord = true;

  // all nodes
  for (unsigned i=1; i!=nodes+1; i++)
  {
    subgraph->add(*(node = newNode()));

    // coodinates
    if (coord)
    {
      double x = xM + radius * cos(2*M_PI/nodes*i);
      double y = yM - radius * sin(2*M_PI/nodes*i);
      node->coord(Coordinates(x,y));
    }
  }

  // all possible edges
  NodeIter end = subgraph->nodesEnd();
  NodeIter beforeEnd = (--subgraph->nodesEnd());
  NodeIter j,i=subgraph->nodesBegin();

  while (i != beforeEnd)
  {
    j = i; j++;
    while (j != end)
      subgraph->add(*newEdge(*i,*j++));

    i++;
  }

  return subgraph;
}

SubGraph* RandomGraph::makeHamilton(unsigned nodes, unsigned edges)
{
  if (nodes > edges)
    THROW(runtime_error,"RandomGraph::makeHamilton(unsigned,unsigned)\
      not enough edges for an Hamiltonian cycle")

  else if (nodes*(nodes-1)/2 < edges)
    THROW(runtime_error,"RandomGraph::makeHamilton(unsigned,unsigned)\
      to much edges !")

  SubGraph* subgraph = new SubGraph(this);
  NodeIter first, source, target;
  subgraph->add(*(first = source = newNode()));

  // hamiltonian cycle
  for (unsigned i=2; i!=nodes+1; i++)
  {
    subgraph->add(*(target = newNode()));
    subgraph->add(*newEdge(*source,*target));
    edges--;
    source = target;
  }
  subgraph->add(*newEdge(*first,*target));
  edges--;

  unsigned v1, v2;

  while(edges)
  {
    v1 = (unsigned)((double)nodes*urn())+1;
    v2 = (unsigned)((double)nodes*urn())+1;

    // if the two random nodes are not same and there is no edge between them
    //  -> a new edge is set
    if (v1!=v2 && findEdge(v1,v2)==edgesEnd())
    {
      subgraph->add(*newEdge(v1,v2));
      edges--;
    }
  }

  return subgraph;
}

SubGraph* RandomGraph::makeRandom(unsigned nodes, unsigned edges)
{
  if (nodes*(nodes-1)/2 < edges)
    THROW(runtime_error,"RandomGraph::makeRandom(unsigned,unsigned)\
      to much edges !")

  SubGraph* subgraph = new SubGraph(this);

  for (unsigned i=1; i!=nodes+1; i++)
    subgraph->add(*newNode());

  unsigned v1, v2;

  while(edges)
  {
    v1 = (unsigned)((double)nodes*urn())+1;
    v2 = (unsigned)((double)nodes*urn())+1;

    // if the two random nodes are not same and there is no edge between them
    //  -> a new edge is set
    if (v1!=v2 && findEdge(v1,v2)==edgesEnd())
    {
      subgraph->add(*newEdge(v1,v2));
      edges--;
    }
  }

  return subgraph;
}


} // namespace graphlib

/* end of file */
